#include "stdafx.h"

#include "PlayVideo.h"

#include <windows.h>
#include <vfw.h>
#include <amvideo.h>

#include "Patcher.h"

TCHAR file[MAX_PATH];     
TCHAR filetitle[MAX_PATH];
 
PAVIFILE pFile;
AVIFILEINFO fileInfo;
 
PAVISTREAM pStream;
AVISTREAMINFO streamInfo;
 
long nFormatSize;
 
BITMAPINFOHEADER* pBmInfoHeaderIn;
BITMAPINFOHEADER pBmInfoHeaderOut;
 
BYTE* decFrame;
BYTE* frame;
double speed; // amount of ms to wait before next frame
double lastFrameTime = 0.0;  // ms
 
RECT rc;
 
HIC hic;
 
long nSampleSize;
int startSample, endSample, curSample;

BOOL isAviFileOpened = FALSE;

class CPatchClass
{
public:

	void swapBufferFunc();

};

CPatch* swapPatcher = NULL;

void CPatchClass::swapBufferFunc()
{
	swapPatcher->remove_patch();
}

BOOL fileExists(const TCHAR * file)
{
   WIN32_FIND_DATA FindFileData;
   HANDLE handle = FindFirstFile(file, &FindFileData) ;
   BOOL found = (handle != INVALID_HANDLE_VALUE);
   if (found) 
   {
       FindClose(handle);
   }
   else
   {
	   CTString strFile(file);
	   CPrintF("Cannot open file %s\n", strFile);
   }
   return found;
}

void AviLoad(const TCHAR *file)
{
	if (isAviFileOpened)
		AviClose();

	if (!(GetSP()->sp_bSinglePlayer || GetSP()->sp_bQuickTest))
		return;

	if (!fileExists(file))
		return;
		
    AVIFileInit();
 
    AVIFileOpen(&pFile, file, OF_READ, 0);
    AVIFileInfo(pFile, &fileInfo, sizeof(AVIFILEINFO));
  
    AVIFileGetStream(pFile, &pStream, streamtypeVIDEO, 0);
    AVIStreamInfo(pStream, &streamInfo, sizeof(AVISTREAMINFO));
    startSample = streamInfo.dwStart;
    endSample = streamInfo.dwStart + streamInfo.dwLength;;
    curSample = startSample;
    nSampleSize = streamInfo.dwSuggestedBufferSize;
    
    AVIStreamReadFormat(pStream, streamInfo.dwStart, NULL, &nFormatSize);
 
    pBmInfoHeaderIn = new BITMAPINFOHEADER[nFormatSize];
    AVIStreamReadFormat(pStream, streamInfo.dwStart, pBmInfoHeaderIn, &nFormatSize);
    
    memcpy(&pBmInfoHeaderOut, pBmInfoHeaderIn, sizeof(BITMAPINFOHEADER));
    pBmInfoHeaderOut.biCompression = BI_RGB;
    pBmInfoHeaderOut.biBitCount = 24;
    pBmInfoHeaderOut.biSizeImage = DIBSIZE(pBmInfoHeaderOut);
 
    decFrame = new BYTE[pBmInfoHeaderOut.biSizeImage];
    memset(decFrame, 0, pBmInfoHeaderOut.biSizeImage);
 
    frame = new BYTE[nSampleSize];
    memset(frame, 0, nSampleSize);
 
    hic = ICDecompressOpen(ICTYPE_VIDEO, pBmInfoHeaderIn->biCompression, 
        pBmInfoHeaderIn, &pBmInfoHeaderOut);
    ICDecompressBegin(hic, pBmInfoHeaderIn, &pBmInfoHeaderOut); 
	
	speed = streamInfo.dwScale * 1000.0 / streamInfo.dwRate;
	lastFrameTime = 0.0;
	isAviFileOpened = TRUE;
} 
 
BOOL AviPlay(CDrawPort* pdp)
{
	if (!isAviFileOpened)
		return FALSE;

	if (_pNetwork->IsPaused() || (_pNetwork->GetRealTimeFactor() == 0.0f))
		return FALSE;
	
	HWND hWnd = pdp->dp_Raster->ra_pvpViewPort->vp_hWnd;
 
    HDC hdc = GetDC(hWnd);
    HDRAWDIB hdd = DrawDibOpen();     

	pdp->Fill(C_BLACK|CT_OPAQUE);

	// Because fuck off
	if (swapPatcher == NULL)
	{		
		void (CViewPort::*pBufferSwapper)(void) = &CViewPort::SwapBuffers;
		swapPatcher = new CPatch(pBufferSwapper, &CPatchClass::swapBufferFunc);
	}
	else
	{
		swapPatcher->set_patch();
	}
	 
    if (curSample <= endSample)
    {
		FLOAT drawPortAspect = (FLOAT)(pdp->GetWidth()) / (FLOAT)(pdp->GetHeight());
		FLOAT pictureAspect = (FLOAT)(pBmInfoHeaderOut.biWidth) / (FLOAT)(pBmInfoHeaderOut.biHeight);
		BOOL bPictureWidthSmaller = (drawPortAspect > pictureAspect);
		INDEX x, y, w, h;
		if (bPictureWidthSmaller)
		{
			y = 0;
			h = pdp->GetHeight();
			w = pBmInfoHeaderOut.biWidth * ((FLOAT)(pdp->GetHeight()) / (FLOAT)(pBmInfoHeaderOut.biHeight));
			x = (pdp->GetWidth() - w) / 2;
		}
		else
		{
			x = 0;
			w = pdp->GetWidth();
			h = pBmInfoHeaderOut.biHeight * ((FLOAT)(pdp->GetWidth()) / (FLOAT)(pBmInfoHeaderOut.biWidth));
			y = (pdp->GetHeight() - h) / 2;
		}

		/*RECT rect;
		GetWindowRect(hWnd, &rect);
		x += rect.left;
		y += rect.top;*/

        AVIStreamRead(pStream, curSample, 1, frame, nSampleSize, NULL, NULL);
        ICDecompress(hic, 0, pBmInfoHeaderIn, frame, &pBmInfoHeaderOut, decFrame);

		// How much time passed?
		double mscurrent = _pTimer->GetHighPrecisionTimer().GetSeconds() * 1000.0;
		double msdesired = lastFrameTime + speed;

		if (msdesired > mscurrent)
			Sleep(msdesired - mscurrent);

		lastFrameTime = _pTimer->GetHighPrecisionTimer().GetSeconds() * 1000.0;		
        DrawDibDraw(hdd, hdc, x, y, w, h, &pBmInfoHeaderOut, decFrame, 0, 0, pBmInfoHeaderOut.biWidth, pBmInfoHeaderOut.biHeight, 0);
 
        ++curSample;

		UpdateWindow(hWnd); // Is this necessary?
		DrawDibClose(hdd);
		ReleaseDC(hWnd, hdc);
		
		//pdp->dp_Raster->ra_pvpViewPort->vp_hWnd = NULL;
		return TRUE;
    }
		
	AviClose();
	return FALSE;
}

void AviClose()
{
	if (!isAviFileOpened)
		return;
	
    ICDecompressEnd(hic);
    ICClose(hic);
 
    AVIStreamRelease(pStream);
    AVIFileRelease(pFile);
    AVIFileExit();
    
    delete [] frame;
    delete [] decFrame;
    delete [] pBmInfoHeaderIn;

	isAviFileOpened = FALSE;
}

BOOL AviIsPlaying()
{
	return isAviFileOpened;
}